#include "beziercurveevaluator.h"
#include <cassert>
using namespace Animator;

//ptvCtrlPts: a collection of control points that you specify in the curve 
//editor
//ptvEvaluatedCurvePts: a collection of evaluated curve points that you 
//return from the function calculated using the curve typefs formulas
//fAniLength: maximum time that a curve is defined
//bWrap: a flag indicating whether or not the curve should be wrapped
void BezierCurveEvaluator::evaluateCurve(const std::vector<Point>& ptvCtrlPts, 
										 std::vector<Point>& ptvEvaluatedCurvePts, 
										 const float& fAniLength, 
										 const bool& bWrap,
										 const float default_val) const
{
	int iCtrlPtCount = ptvCtrlPts.size();
	ptvEvaluatedCurvePts.clear();

	// For every set of four points, specify several Bezier curve points
	if (iCtrlPtCount < 4)
	{
		// linear
	}
	else
	{
		Point v0, v1, v2, v3;
		int pointIndex = 0;

		v0 = ptvCtrlPts[pointIndex];
		v1 = ptvCtrlPts[pointIndex+1];
		v2 = ptvCtrlPts[pointIndex+2];
		v3 = ptvCtrlPts[pointIndex+3];

		float step = 0.05f;
		for (float u = 0.0f; u <= 1.0f + 0.0001; u+=step)
		{
			// Use de Casteljau's algorithm to get points on the curve
			Point newPoint;
			newPoint.x = pow(1-u, 3) * v0.x + 3 * u * pow(1-u,2) * v1.x + 3 * pow(u, 2) * (1-u) * v2.x + pow(u,3) * v3.x;
			newPoint.y = pow(1-u, 3) * v0.y + 3 * u * pow(1-u,2) * v1.y + 3 * pow(u, 2) * (1-u) * v2.y + pow(u,3) * v3.y;
			ptvEvaluatedCurvePts.push_back(newPoint);
		}

		pointIndex = 4;

		while(pointIndex + 3 <= iCtrlPtCount)
		{
			v0 = ptvCtrlPts[pointIndex-1];
			v1 = ptvCtrlPts[pointIndex];
			v2 = ptvCtrlPts[pointIndex+1];
			v3 = ptvCtrlPts[pointIndex+2];
			
			float step = 0.05f;
			for (float u = 0.0f; u <= 1.0f + 0.0001; u+=step)
			{
				// Use de Casteljau's algorithm to get points on the curve
				Point newPoint;
				newPoint.x = pow(1-u, 3) * v0.x + 3 * u * pow(1-u,2) * v1.x + 3 * pow(u, 2) * (1-u) * v2.x + pow(u,3) * v3.x;
				newPoint.y = pow(1-u, 3) * v0.y + 3 * u * pow(1-u,2) * v1.y + 3 * pow(u, 2) * (1-u) * v2.y + pow(u,3) * v3.y;
				ptvEvaluatedCurvePts.push_back(newPoint);
			}

			pointIndex += 3;
		}
		// For the remaining (fewer than four) points, just put them in as linear points,
		// since they can't be used to form another Bezier curve
		while(pointIndex < iCtrlPtCount)
		{
			ptvEvaluatedCurvePts.push_back(ptvCtrlPts[pointIndex]);
			++pointIndex;
		}
		
	}
	

	float x = 0.0;
	float y1;

	if (bWrap) {
		// if wrapping is on, interpolate the y value at xmin and
		// xmax so that the slopes of the lines adjacent to the
		// wraparound are equal.

		if ((ptvCtrlPts[0].x + fAniLength) - ptvCtrlPts[iCtrlPtCount - 1].x > 0.0f) {
			y1 = (ptvCtrlPts[0].y * (fAniLength - ptvCtrlPts[iCtrlPtCount - 1].x) + 
				  ptvCtrlPts[iCtrlPtCount - 1].y * ptvCtrlPts[0].x) /
				 (ptvCtrlPts[0].x + fAniLength - ptvCtrlPts[iCtrlPtCount - 1].x);
		}
		else 
			y1 = ptvCtrlPts[0].y;
	}
	else {
		// if wrapping is off, make the first and last segments of
		// the curve horizontal.

		y1 = ptvCtrlPts[0].y;
    }

	ptvEvaluatedCurvePts.push_back(Point(x, y1));

	/// set the endpoint based on the wrap flag.
	float y2;
    x = fAniLength;
    if (bWrap)
		y2 = y1;
    else
		y2 = ptvCtrlPts[iCtrlPtCount - 1].y;

	ptvEvaluatedCurvePts.push_back(Point(x, y2));
}
