#include "catmullromcurveevaluator.h"
#include <cassert>
using namespace Animator;

//ptvCtrlPts: a collection of control points that you specify in the curve 
//editor
//ptvEvaluatedCurvePts: a collection of evaluated curve points that you 
//return from the function calculated using the curve typefs formulas
//fAniLength: maximum time that a curve is defined
//bWrap: a flag indicating whether or not the curve should be wrapped
void CatmullRomCurveEvaluator::evaluateCurve(const std::vector<Point>& ptvCtrlPts, 
										 std::vector<Point>& ptvEvaluatedCurvePts, 
										 const float& fAniLength, 
										 const bool& bWrap,
										 const float default_val) const
{
	int iCtrlPtCount = ptvCtrlPts.size();
	ptvEvaluatedCurvePts.clear();

	// Curve tension (tau)
	float tension = 0.5f;
	
	// Modify set of control points to Bezier form via C-R rules
	std::vector<Point> bezierCtrlPts;
	bezierCtrlPts.clear();

	std::vector<Point> myCtrlPoints(ptvCtrlPts);

	// Duplicate first control point for Bezier control points
	myCtrlPoints.insert(myCtrlPoints.begin(), ptvCtrlPts.front());

	// Duplicate last control point twice for Bezier control points
	myCtrlPoints.push_back(ptvCtrlPts.back());

	// For every set of four points, use them to calculate points to add to the bezier control points
	int pointIndex = 0;
	Point v0, v1, v2, v3;
	while(pointIndex + 3 < myCtrlPoints.size())
	{
		Point p0 = myCtrlPoints[pointIndex];
		Point p1 = myCtrlPoints[pointIndex+1];
		Point p2 = myCtrlPoints[pointIndex+2];
		Point p3 = myCtrlPoints[pointIndex+3];

		v0.x = p1.x;
		v1.x = p1.x + (tension / 3.f) * (p2.x - p0.x);
		v2.x = p2.x - (tension / 3.f) * (p3.x - p1.x);
		v3.x = p2.x;

		v0.y = p1.y;
		v1.y = p1.y + (tension / 3.f) * (p2.y - p0.y);
		v2.y = p2.y - (tension / 3.f) * (p3.y - p1.y);
		v3.y = p2.y;

		// Correct the v1 and v2 if they happen to fall before or
		// after v0 and v3 in terms of x-values (so that curves
		// behave monotonically increasing along x)
		if (v1.x < v0.x)
			v1.x = v0.x;
		else if (v1.x > v3.x)
			v1.x = v3.x;
		if (v2.x > v3.x)
			v2.x = v3.x;
		else if (v2.x < v0.x)
			v2.x = v0.x;
		if (v1.x > v2.x)
		{
			v1.x = v2.x;
		}

		float step = 0.05f;
		for (float u = 0.0f; u <= 1.0f + 0.0001; u+=step)
		{
			// Use de Casteljau's algorithm to get points on the curve
			Point newPoint;
			newPoint.x = pow(1-u, 3) * v0.x + 3 * u * pow(1-u,2) * v1.x + 3 * pow(u, 2) * (1-u) * v2.x + pow(u,3) * v3.x;
			newPoint.y = pow(1-u, 3) * v0.y + 3 * u * pow(1-u,2) * v1.y + 3 * pow(u, 2) * (1-u) * v2.y + pow(u,3) * v3.y;
			ptvEvaluatedCurvePts.push_back(newPoint);
		}
		pointIndex++;
	}

	float x = 0.0;
	float y1;

	if (bWrap) {
		// if wrapping is on, interpolate the y value at xmin and
		// xmax so that the slopes of the lines adjacent to the
		// wraparound are equal.

		if ((ptvCtrlPts[0].x + fAniLength) - ptvCtrlPts[iCtrlPtCount - 1].x > 0.0f) {
			y1 = (ptvCtrlPts[0].y * (fAniLength - ptvCtrlPts[iCtrlPtCount - 1].x) + 
				  ptvCtrlPts[iCtrlPtCount - 1].y * ptvCtrlPts[0].x) /
				 (ptvCtrlPts[0].x + fAniLength - ptvCtrlPts[iCtrlPtCount - 1].x);
		}
		else 
			y1 = ptvCtrlPts[0].y;
	}
	else {
		// if wrapping is off, make the first and last segments of
		// the curve horizontal.

		y1 = ptvCtrlPts[0].y;
    }

	ptvEvaluatedCurvePts.push_back(Point(x, y1));

	/// set the endpoint based on the wrap flag.
	float y2;
    x = fAniLength;
    if (bWrap)
		y2 = y1;
    else
		y2 = ptvCtrlPts[iCtrlPtCount - 1].y;

	ptvEvaluatedCurvePts.push_back(Point(x, y2));
}
