#include "model.h"
#include <algorithm>
#include "modelerdraw.h"
#include "mat.h"
#include <time.h>
#include "modelerui.h"

using namespace std;

///////////// class Model ///////////////
Model::Model(const char* name) : properties(name) {}
GroupProperty* Model::getProperties() { return &properties; }
void Model::draw() {}
void Model::tick() {}
void Model::load() {}
ICamera* Model::getCamera() { return NULL; }
ParticleSystem* Model::getParticleSystem() { return NULL; };

Mat4f getModelViewMatrix();

///////////// class Light ///////////////
Light::Light(const char* name, GLuint lightNumber,
						 float x, float y, float z,
						 float dr, float dg, float db,
						 float sr, float sg, float sb,
						 float ar, float ag, float ab) :
	Model(name),
    lightX("Light X", -10, 10, x, .1f),
	lightY("Light Y", -10, 10, y, .1f),
	lightZ("Light Z", -10, 10, z, .1f),
	diffuse("Diffuse", dr, dg, db),
	specular("Specular", sr, sg, sb),
	ambient("Ambient",  ar, ag, ab),
	isOn("This light is on", true)
	{

	properties.add(&isOn)
			  .add(&diffuse)
			  .add(&specular)
			  .add(&ambient)
			  .add(&lightX)
			  .add(&lightY)
			  .add(&lightZ);
	
	properties.setCollapsed(true);

	this->lightNumber = lightNumber;
}

void Light::draw(bool directional) {
	if (!isOn.getValue()) {
		// Set values to 0
		const float zeroes[4] = {0,0,0,0};
		glLightfv( lightNumber, GL_DIFFUSE, zeroes );
		glLightfv( lightNumber, GL_SPECULAR, zeroes );
		glLightfv( lightNumber, GL_AMBIENT, zeroes );
		return;
	}

	// Enable OpenGL lighting
	// TODO: check to see if it's already on
	glEnable( GL_LIGHTING );

	// Enable the light we're using
	// TODO: check to see if it's already on
	glEnable(lightNumber);

	// Create array containing light position
	GLfloat lightPositionArray[4] = {
		lightX.getValue(),
		lightY.getValue(),
		lightZ.getValue(),
		directional ? 0.f : 1.f
	};

	// Return if the light is directional but has no direction.
	if (directional && lightPositionArray[0] == 0
		&& lightPositionArray[1] == 0
		&& lightPositionArray[2] == 0) return;

	// Create array containing light diffuse color
	GLfloat lightDiffuseArray[4] = {
		diffuse.getRed(),
		diffuse.getGreen(),
		diffuse.getBlue(),
		1
	};

	// Create array for light's specular color
	GLfloat lightSpecularArray[4] = {
		specular.getRed(),
		specular.getGreen(),
		specular.getBlue(),
		1
	};

	// Create array for light's ambient color
	GLfloat lightAmbientArray[4] = {
		ambient.getRed(),
		ambient.getGreen(),
		ambient.getBlue(),
		1
	};

	// Send light to OpenGL
	glLightfv( lightNumber, GL_POSITION, lightPositionArray );
	glLightfv( lightNumber, GL_DIFFUSE, lightDiffuseArray );
	glLightfv( lightNumber, GL_SPECULAR, lightSpecularArray );
	glLightfv( lightNumber, GL_AMBIENT, lightAmbientArray );

	// Draw a marker for the light if allowed
	if (ModelerDrawState::Instance()->showMarkers) {
		const float lightRadius = .5f;
		const float directionalOrbit = 5.f;
		
		glPushMatrix();
			glTranslatef(lightPositionArray[0],
						 lightPositionArray[1],
						 lightPositionArray[2]);
			glDisable(GL_LIGHTING);
			glDisable(GL_TEXTURE_2D);
			glColor3f(1, 1, 1);
			glBegin(GL_LINES);
				glVertex3f(-lightRadius, 0, 0);
				glVertex3f(lightRadius, 0, 0);
				glVertex3f(0, -lightRadius, 0);
				glVertex3f(0, lightRadius, 0);
				glVertex3f(0, 0, -lightRadius);
				glVertex3f(0, 0, lightRadius);
			glEnd();
			glEnable(GL_LIGHTING);
		glPopMatrix();
	}
}

PointLight::PointLight(const char* name, GLuint lightNumber,
				float x, float y, float z,
				float dr, float dg, float db,
				float sr, float sg, float sb,
				float ar, float ag, float ab,
				float attA, float attB, float attC) :
	Light(name, lightNumber, x, y, z, dr, dg, db, sr, sg, sb, ar, ag, ab),
	attenA("Quadratic Attenuation ^ 5", 0, 1, attA, .05f),
	attenB("Linear Attenuation ^ 5", 0, 1, attB, .05f),
	attenC("Constant Attenuation ^ 5", 0, 1, attC, .05f)
{
	properties.add(&attenA).add(&attenB).add(&attenC);
}

void PointLight::draw() {
	// Set attenuation
	glLightf(lightNumber, GL_CONSTANT_ATTENUATION, pow(attenC.getValue(), 5));
	glLightf(lightNumber, GL_LINEAR_ATTENUATION, pow(attenB.getValue(), 5));
	glLightf(lightNumber, GL_QUADRATIC_ATTENUATION, pow(attenA.getValue(), 5));

	Light::draw(false);
}

/**
 * A directional light.
 */
DirectionalLight::DirectionalLight(const char* name, GLuint lightNumber,
				float x, float y, float z,
				float dr, float dg, float db,
				float sr, float sg, float sb) :
	Light(name, lightNumber, x, y, z, dr, dg, db, sr, sg, sb)
{
}

void DirectionalLight::draw() {
	Light::draw(true);
}

/* 
Mars Curiosity Rover
*/
CuriosityRover::CuriosityRover(const char* name, GLdouble x, GLdouble y, GLdouble z, GLdouble sizeVal) :
	Model(name),
	size("Size", 1.0, 8.0, sizeVal, 0.1),
	chassisTex("chassisTex.png"),
	bogieTex("bogieTex.png"),
	wheelTex("wheelTex.png"),
	RTGTex("RTGTex.png"),
	cameraMastTex("cameraMastTex.png"),
	cameraTex0("cameraTex0.png"),
	cameraTex1("cameraTex1.png"),
	cameraTex2("cameraTex2.png"),
	cameraTex3("cameraTex3.png"),
	cameraTex4("cameraTex4.png"),
	cameraTex5("cameraTex5.png"),
	cameraTex6("cameraTex6.png"),
	cameraTex7("cameraTex7.png"),
	cameraTex8("cameraTex8.png"),
	armTex("armTex.png"),
	handTex("handTex.png"),
	roverX("Rover X", -50.f, 50.f, 0.f, 0.1f),
	roverY("Rover Y", -50.f, 50.f, 0.f, 0.1f),
	roverZ("Rover Z", -50.f, 50.f, 0.f, 0.1f),
	portFrontRodRotation("Port front rod rotation", 0.0f, 90.f, 15.f, 1.f),
	portCenterRodRotation("Port center rod rotation", 0.0f, 90.f, 15.f, 1.f),
	portBackFrontRodRotation("Port back-front rod rotation", 0.0f, 90.f, 50.f, 1.f),
	portBackBackRodRotation("Port back-back rod rotation", 0.0f, 90.f, 10.f, 1.f),
	starboardFrontRodRotation("Starboard front rod rotation", 0.0f, 90.f, 15.f, 1.f), 
	starboardCenterRodRotation("Starboard center rod rotation", 0.0f, 90.f, 15.f, 1.f), 
	starboardBackFrontRodRotation("Starboard back-front rod rotation", 0.0f, 90.f, 50.f, 1.f), 
	starboardBackBackRodRotation("Starboard back-back rod rotation", 0.0f, 90.f, 10.f, 1.f),
	portFrontWheelSwivel("Port front wheel swivel", 0.0f, 90.f, 45.f, 1.f),
	portCenterWheelSwivel("Port center wheel swivel", 0.0f, 90.f, 45.f, 1.f), 
	portBackWheelSwivel("Port back wheel swivel", 0.0f, 90.f, 45.f, 1.f), 
	starboardFrontWheelSwivel("Starboard front wheel swivel", 0.0f, 90.f, 45.f, 1.f), 
	starboardCenterWheelSwivel("Starboard center wheel swivel", 0.0f, 90.f, 45.f, 1.f), 
	starboardBackWheelSwivel("Starboard back wheel swivel", 0.0f, 90.f, 45.f, 1.f), 
	portFrontWheelSpin("Port front wheel spin", 0.0f, 360.f, 0.f, 1.f),
	portCenterWheelSpin("Port center wheel spin", 0.0f, 360.f, 0.f, 1.f),
	portBackWheelSpin("Port back wheel spin", 0.0f, 360.f, 0.f, 1.f),
	starboardFrontWheelSpin("Starboard front wheel spin",0.0f, 360.f, 0.f, 1.f),
	starboardCenterWheelSpin("Starboard center wheel spin", 0.0f, 360.f, 0.f, 1.f),
	starboardBackWheelSpin("Starboard back wheel spin", 0.0f, 360.f, 0.f, 1.f),
	globalWheelSpin("Global wheel spin", 0.0f, 36000.f, 36000.f, 1.f),
	cameraMastSwivel("Camera mast swivel", 0.0f, 180.f, 90.f, 1.f),
	cameraTilt("Camera tilt", -40.0f, 70.f, 0.f, 1.f),
	cameraBlink("Camera blink", 0, 8, 0, 1),
	upperArmSwivel("Upper arm swivel", 0.0f, 180.f, 90.f, 1.f),
	upperArmTilt("Upper arm tilt", 0.0f, 90.f, 90.f, 1.f),
	lowerArmTilt("Lower arm tilt", -0.f, 90.f, 0.0f, 1.f),
	handTilt("Hand tilt", -30.0f, 30.f, 0.f, 1.f),
	firingLaser("Firing Laser", 0.f, 1.f, 0.f, 0.1f),
	dusting("Dusting",  0.f, 1.f, 0.f, 0.1f),
	ps()
{
    this->x = x;
    this->y = y;
    this->z = z;
	this->name = name;

	properties.add(&size);
	properties.add(&roverX);
	properties.add(&roverY);
	properties.add(&roverZ);
	properties.add(&portFrontRodRotation);
	properties.add(&portCenterRodRotation);
	properties.add(&portBackFrontRodRotation);
	properties.add(&portBackBackRodRotation);
	properties.add(&starboardFrontRodRotation);
	properties.add(&starboardCenterRodRotation);
	properties.add(&starboardBackFrontRodRotation);
	properties.add(&starboardBackBackRodRotation);
	properties.add(&portFrontWheelSwivel);
	properties.add(&portCenterWheelSwivel);
	properties.add(&portBackWheelSwivel);
	properties.add(&starboardFrontWheelSwivel);
	properties.add(&starboardCenterWheelSwivel);
	properties.add(&starboardBackWheelSwivel);
	properties.add(&portFrontWheelSpin);
	properties.add(&portCenterWheelSpin);
	properties.add(&portBackWheelSpin);
	properties.add(&starboardFrontWheelSpin);
	properties.add(&starboardCenterWheelSpin);
	properties.add(&starboardBackWheelSpin);
	properties.add(&globalWheelSpin);
	properties.add(&cameraMastSwivel);
	properties.add(&cameraTilt);
	properties.add(&cameraBlink);
	properties.add(&upperArmSwivel);
	properties.add(&upperArmTilt);
	properties.add(&lowerArmTilt);
	properties.add(&handTilt);
	properties.add(&firingLaser);
	properties.add(&dusting);
	properties.add(&ps.restitution);
	properties.add(&ps.friction);


}

void CuriosityRover::load()
{
	chassisTex.load();
	bogieTex.load();
	wheelTex.load();
	RTGTex.load();
	cameraTex0.load();
	cameraTex1.load();
	cameraTex2.load();
	cameraTex3.load();
	cameraTex4.load();
	cameraTex5.load();
	cameraTex6.load();
	cameraTex7.load();
	cameraTex8.load();
	cameraMastTex.load();
	armTex.load();
	handTex.load();
}

void CuriosityRover::draw()
{
	Mat4f cameraTransformationMatrix = getModelViewMatrix();

	glPushMatrix();

		// Draw main body
		drawChassis();

		// Draw wheels, etc.
		drawRockerBogies(cameraTransformationMatrix);

		// Draw camera mast
		drawCameraMast();

		// Draw RTG
		drawRTG();

		// Draw arm
		drawArm(cameraTransformationMatrix);

		glBindTexture(GL_TEXTURE_2D, 0);

	glPopMatrix();

	ModelerUserInterface* ui = ModelerUserInterface::getInstance();
	ps.computeForcesAndUpdateParticles(ui->getTime());
	ps.drawParticles(ui->getTime());
	ps.cleanup();

}

void CuriosityRover::drawChassis()
{
	chassisTex.use();

    // Draw a rectangular prism to serve as the chassis
    
    // Put on the ground
    //glTranslatef(-580.f, -size.getValue() * 0.4f, -952.f);
	glTranslatef(roverX.getValue(), roverY.getValue() + -size.getValue() * 0.4f, roverZ.getValue());

    glPushMatrix();
    // Make dimensions 4 x 1 x 3
    glScalef(1.33, 0.33, 1.0);

	drawBox(size.getValue(), size.getValue(), size.getValue());

    glPopMatrix();

}


void CuriosityRover::drawRockerBogies(Mat4f cameraTransformationMatrix)
{
	//
	/** Draw port side rocket bogie **/
    //

	// Far front rod
	glPushMatrix();
		bogieTex.use();
		glTranslatef(-size.getValue() * 0.15, -size.getValue() * 0.07, size.getValue() * 0.55);
		glRotatef(-90.f, 0.0f, 1.0f, 0.0f);
		glRotatef(portFrontRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
		drawCylinder(size.getValue() * 0.75, size.getValue() * 0.03, size.getValue() * 0.03);
		
		glTranslatef(size.getValue() * 0.05, 0.0f, size.getValue() * 0.75);
		glRotatef(-15.f, 1.0f, 0.0f, 0.0f);
		glRotatef(135 - portFrontWheelSwivel.getValue(), 0.0f, 1.0f, 0.0f);

		// Spin
		glRotatef(-portFrontWheelSpin.getValue() - globalWheelSpin.getValue(), 0.0f, 0.0f, 1.0f);

		drawWheel(cameraTransformationMatrix);

	glPopMatrix();

	// Center rod
	glPushMatrix();
		bogieTex.use();
		glTranslatef(-size.getValue() * 0.15, -size.getValue() * 0.07, size.getValue() * 0.55);
		glRotatef(90.f, 0.0f, 1.0f, 0.0f);
		glRotatef(portCenterRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
		drawCylinder(size.getValue() * 0.55, size.getValue() * 0.03, size.getValue() * 0.03);

		// Back-front rod
		glPushMatrix();
			bogieTex.use();
			glTranslatef(0.0f, 0.0f, size.getValue() * 0.55);
			glRotatef(90.f + portBackFrontRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
			drawCylinder(size.getValue() * 0.35, size.getValue() * 0.03, size.getValue() * 0.03);

			glTranslatef(-size.getValue() * 0.05, 0.0f, size.getValue() * 0.35);

			glRotatef(-140.f, 1.0f, 0.0f, 0.0f);
			glRotatef(-15.f, 1.0f, 0.0f, 0.0f);		
			glRotatef(-45 - portCenterWheelSwivel.getValue(), 0.0f, 1.0f, 0.0f);

			// Spin
			glRotatef(-portCenterWheelSpin.getValue() - globalWheelSpin.getValue(), 0.0f, 0.0f, 1.0f);

			drawWheel(cameraTransformationMatrix);
		glPopMatrix();

		// Back-back rod
		glPushMatrix();
			bogieTex.use();
			glTranslatef(0.0f, 0.0f, size.getValue() * 0.55);
			glRotatef(-portBackBackRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
			drawCylinder(size.getValue() * 0.35, size.getValue() * 0.03, size.getValue() * 0.03);

			glTranslatef(-size.getValue() * 0.05, 0.0f, size.getValue() * 0.35);
			
			glRotatef(10.f, 1.0f, 0.0f, 0.0f);
			glRotatef(-15.f, 1.0f, 0.0f, 0.0f);		
			glRotatef(-45 - portBackWheelSwivel.getValue(), 0.0f, 1.0f, 0.0f);
			
			// Spin
			glRotatef(-portBackWheelSpin.getValue() - globalWheelSpin.getValue(), 0.0f, 0.0f, 1.0f);

			drawWheel(cameraTransformationMatrix);
		glPopMatrix();

	glPopMatrix();


    //
	/** Draw starboard side rocket bogie **/
    //

	// Far front rod
	glPushMatrix();
		bogieTex.use();
		glTranslatef(-size.getValue() * 0.15, -size.getValue() * 0.07, -size.getValue() * 0.55);
		glRotatef(-90.f, 0.0f, 1.0f, 0.0f);
		glRotatef(starboardFrontRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
		drawCylinder(size.getValue() * 0.75, size.getValue() * 0.03, size.getValue() * 0.03);
		
		// Wheel swivel
		glTranslatef(-size.getValue() * 0.36, 0.0f, size.getValue() * 0.75);
		glRotatef(-15.f, 1.0f, 0.0f, 0.0f);
		glRotatef(135 - starboardFrontWheelSwivel.getValue(), 0.0f, 1.0f, 0.0f);

		// Wheel spin
		glRotatef(-starboardFrontWheelSpin.getValue() - globalWheelSpin.getValue(), 0.0f, 0.0f, 1.0f);

		drawWheel(cameraTransformationMatrix);

	glPopMatrix();

	// Center rod
	glPushMatrix();
		bogieTex.use();
		glTranslatef(-size.getValue() * 0.15, -size.getValue() * 0.07, -size.getValue() * 0.55);
		glRotatef(90.f, 0.0f, 1.0f, 0.0f);
		glRotatef(starboardCenterRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
		drawCylinder(size.getValue() * 0.55, size.getValue() * 0.03, size.getValue() * 0.03);

		// Back-front rod
		glPushMatrix();
			glTranslatef(0.0f, 0.0f, size.getValue() * 0.55);
			glRotatef(90.f + starboardBackFrontRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
			drawCylinder(size.getValue() * 0.35, size.getValue() * 0.03, size.getValue() * 0.03);

			glTranslatef(size.getValue() * 0.36, 0.0f, size.getValue() * 0.35);
			glRotatef(-140.f, 1.0f, 0.0f, 0.0f);
			glRotatef(-15.f, 1.0f, 0.0f, 0.0f);		
			glRotatef(-45 - starboardCenterWheelSwivel.getValue(), 0.0f, 1.0f, 0.0f);

			// Spin
			glRotatef(-starboardCenterWheelSpin.getValue() - globalWheelSpin.getValue(), 0.0f, 0.0f, 1.0f);

			drawWheel(cameraTransformationMatrix);
		glPopMatrix();

		// Back-back rod
		glPushMatrix();
			bogieTex.use();
			glTranslatef(0.0f, 0.0f, size.getValue() * 0.55);
			glRotatef(-starboardBackBackRodRotation.getValue(), 1.0f, 0.0f, 0.0f);
			drawCylinder(size.getValue() * 0.35, size.getValue() * 0.03, size.getValue() * 0.03);
		
			glTranslatef(size.getValue() * 0.05, 0.0f, size.getValue() * 0.35);
			glRotatef(10.f, 1.0f, 0.0f, 0.0f);
			glRotatef(-15.f, 1.0f, 0.0f, 0.0f);
			glRotatef(135 - starboardBackWheelSwivel.getValue(), 0.0f, 1.0f, 0.0f);

			// Spin
			glRotatef(starboardBackWheelSpin.getValue() + globalWheelSpin.getValue(), 0.0f, 0.0f, 1.0f);

			drawWheel(cameraTransformationMatrix);

		glPopMatrix();

	glPopMatrix();

}

void CuriosityRover::drawWheel(Mat4f cameraTransformationMatrix)
{
	wheelTex.use();
	glPushMatrix();

		drawCylinder(size.getValue() * 0.3, size.getValue() * 0.3, size.getValue() * 0.3);

		ParticleSystem* myPs = getParticleSystem();

		
		if ((dusting.getValue() > 0.5f)&&(myPs->isRunning()))
		{
			// Occasionally spawn a dust particle
			if ((float)rand()/(float)RAND_MAX < 0.10f)
			{
				spawnParticles(cameraTransformationMatrix, 0);
				//printf("dust particle spawned!\n");
			}
		}

	glPopMatrix();

}

void CuriosityRover::drawCameraMast()
{
	glPushMatrix();
		cameraMastTex.use();
		
		// Move into front starboard corner
		glTranslatef(-size.getValue() * 0.45, 0.0f, -size.getValue() * 0.30);
	
		// Rotate it up
		glRotatef(-90.f, 1.0f, 0.0f, 0.0f);

		// Camera mast swivel
		glRotatef(cameraMastSwivel.getValue() - 90.f, 0.0f, 0.0f, 1.0f);

		// Draw main mast
		drawCylinder(size.getValue(), size.getValue() * 0.05, size.getValue() * 0.05);

		// Go up to top of mast
		glTranslatef(0.0f, 0.0f, size.getValue());

		// Camera tilt
		glRotatef(cameraTilt.getValue(), 0.0f, 1.0f, 0.0f);

		// Draw camera top
		switch((int)cameraBlink.getValue())
		{
		case 0:
			cameraTex0.use();
			break;
		case 1:
			cameraTex1.use();
			break;
		case 2:
			cameraTex2.use();
			break;
		case 3:
			cameraTex3.use();
			break;
		case 4:
			cameraTex4.use();
			break;
		case 5:
			cameraTex5.use();
			break;
		case 6:
			cameraTex6.use();
			break;
		case 7:
			cameraTex7.use();
			break;
		case 8:
			cameraTex8.use();
			break;
		default:
			cameraTex0.use();
		}
		

		glScalef(0.3f, 0.4f, 0.3f);
		drawBox(size.getValue(), size.getValue(), size.getValue());
	
	glPopMatrix();

}

void CuriosityRover::drawRTG()
{
	glPushMatrix();
		RTGTex.use();

		// Move into back center location
		glTranslatef(size.getValue() * 0.70f, size.getValue() * 0.2f, 0.0f);

		// Rotate into place
		glRotatef(30.f, 0.0f, 0.0f, 1.0f);
		
		// Scale to correct size
		glScalef(0.6f, 0.3f, 0.6f);

		drawBox(size.getValue(), size.getValue(), size.getValue());

	glPopMatrix();
}

void CuriosityRover::drawArm(Mat4f cameraTransformationMatrix)
{
	glPushMatrix();
		
		armTex.use();
		
		// Move to front of rover
		glTranslatef(-size.getValue() * 0.70f, 0.0f, size.getValue() * 0.3f);
	
		// Apply swivel then draw small swivel base
		glRotatef(-upperArmSwivel.getValue(), 0.0f, 1.0f, 0.0f);
		drawCylinder(size.getValue() * 0.2, size.getValue() * 0.05, size.getValue() * 0.05);

		// Move to end of swivel base
		glTranslatef(0.0f, 0.0f, size.getValue() * 0.2);
				
		// Apply upper arm tilt
		glRotatef(-upperArmTilt.getValue(), 1.0f, 0.0f, 0.0f);

		// Draw upper arm
		drawCylinder(size.getValue() * 0.5, size.getValue() * 0.05, size.getValue() * 0.05);

		// Go up to top of upper arm
		glTranslatef(0.0f, 0.0f, size.getValue() * 0.5);

		// Rotate and draw lower arm
		glRotatef(lowerArmTilt.getValue(), 1.0f, 0.0f, 0.0f);
		drawCylinder(size.getValue() * 0.5, size.getValue() * 0.05, size.getValue() * 0.05);

		// Go up to top of lower arm
		glTranslatef(0.0f, 0.0f, size.getValue() * 0.5);

		handTex.use();
		glScalef(0.3f, 0.3f, 0.3f);

		// Apply hand tilt and draw
		glRotatef(90 - handTilt.getValue(), 1.0f, 0.0f, 0.0f);
		drawCylinder(size.getValue(), size.getValue() * 0.75, size.getValue() * 0.25);

		// Particles are spawned from the tip of the cylinder
		glTranslatef(0.f, 0.f, size.getValue());
		if ((firingLaser.getValue() > 0.5f) && (getParticleSystem()->isRunning()))
		{
			// Lasers (can) make lots of particles!
			int burstSize = 1;
			for (int i = 0; i < burstSize; i++)
			{
				spawnParticles(cameraTransformationMatrix, 1);
			}
		}

		// drawBox(size.getValue(), size.getValue(), size.getValue());

	glPopMatrix();
}

Mat4f getModelViewMatrix()
{
	/**************************
	**
	**	GET THE OPENGL MODELVIEW MATRIX
	**
	**	Since OpenGL stores its matrices in
	**	column major order and our library
	**	uses row major order, we will need to
	**	transpose what OpenGL gives us before returning.
	**
	**	Hint:  Use look up glGetFloatv or glGetDoublev
	**	for how to get these values from OpenGL.
	**
	*******************************/

        GLfloat m[16];
        glGetFloatv(GL_MODELVIEW_MATRIX, m);
        Mat4f matMV(m[0], m[1], m[2], m[3],
            m[4], m[5], m[6], m[7],
            m[8], m[9], m[10], m[11],
            m[12], m[13], m[14], m[15] );

        return matMV.transpose(); // convert to row major order
}


void CuriosityRover::spawnParticles(Mat4f cameraTransformationMatrix, int type)
{
	/****************************************************************
	**
	**	THIS FUNCTION WILL ADD A NEW PARTICLE TO OUR WORLD
	**
	**	Suppose we want particles to spawn from the model's arm.
	**	We need to find the location of the model's arm in world
	**  coordinates so that we can set the initial position of new
	**  particles.	As discussed on the Animator project page, 
	**  all particle positions should be in world coordinates. 
	**
	**  At this point in execution, the MODELVIEW matrix contains the 
	**  camera transforms multiplied by some model transforms.  In other words, 
	**
	**  MODELVIEW = CameraTransforms * ModelTransforms
	**
	**	We are interested only in ModelTransforms, which is the 
	**  transformation that will convert a point from the current, local 
	**  coordinate system to the world coordinate system.
	**
	**	To do this, we're going to "undo" the camera transforms from the 
	**  current MODELVIEW matrix.  The camera transform is passed in as 
	**  a parameter to this function (remember when we saved it 
	**  near the top of the model's draw method?).  We can "undo" the
	**  camera transforms by pre-multiplying the current MODELVIEW matrix 
	**  with the inverse of the camera matrix.  In other words,
	**
	**  ModelTransforms = InverseCameraTransforms * MODELVIEW
	**    
	********************************************************************/	

	//... Get the current MODELVIEW matrix.
	//... "Undo" the camera transforms from the MODELVIEW matrix
	//... by multiplying Inverse( CameraTransforms ) * CurrentModelViewMatrix.
	//... Store the result of this in a local variable called WorldMatrix.
	//...
	Mat4f currentModelViewMatrix = getModelViewMatrix();

	Mat4f worldMatrix = cameraTransformationMatrix.inverse() * currentModelViewMatrix;
	
	/*****************************************************************
	**
	**	At this point, we have the transformation that will convert a point 
	**  in the local coordinate system to a point in the world coordinate 
	**  system.  
	**
	**  We need to find the actual point in world coordinates 
	**  where particle should be spawned.  This is simply 
	**  "the origin of the local coordinate system" transformed by 
	**  the WorldMatrix.
	**
	******************************************************************/
	Vec4f worldPoint = worldMatrix * Vec4f(0.f, 0.f, 0.f, 1.f);
	
	Vec3f vel;
	if (type == 1)
	{
		// Determine the current velocity vector by going further in
		// our local Z direction, then comparing that world point to
		// our previous one
		glTranslatef(0.f, 0.f, 1.f);
	
		// Convert to world point
		currentModelViewMatrix = getModelViewMatrix();
		worldMatrix = cameraTransformationMatrix.inverse() * currentModelViewMatrix;
		Vec4f nextWorldPoint = worldMatrix * Vec4f(0.f, 0.f, 0.f, 1.f);

		// Get vector from first world point to the next one
		Vec4f displacement = nextWorldPoint - worldPoint;

		glTranslatef(0.f, 0.f, -1.f); // Translate back

		// Normal displacement vector to get the direction
		Vec3f direction(displacement[0], displacement[1], displacement[2]);
		direction.normalize();

		// Finally, assign velocity to the direction * speed
		float speed = 16.f;
		vel = direction * speed;
	}
	else
	{
		// Dust--the velocity should just be up with a random perturbation
		Vec3f direction((float)rand()/(float)RAND_MAX * 0.5f, 1.f, (float)rand()/(float)RAND_MAX * 0.5f);
		direction.normalize();
		float speed = 6.f;
		vel = direction * speed;
	}

	addParticleStartingAt(worldPoint, type, vel);

}

void CuriosityRover::addParticleStartingAt(Vec4f startingPoint, int type, Vec3f vel)
{
	
	// Adds a particle at this point
	
	// While testing, just add a particle at a random offset
	float x = startingPoint[0];
	float y = startingPoint[1];
	float z = startingPoint[2];
	float w = startingPoint[3];

	Vec4f pos(x, y, z, w);

	float mass = 0.1f;

	Particle newParticle;

	newParticle.position = pos;
	newParticle.velocity = vel;
	newParticle.mass = mass;
	newParticle.lastUpdateTime = -1.f; // not yet updated!
	newParticle.type = type;

	// Set random texture coordinates
	newParticle.s = (float)rand()/(float)RAND_MAX;
	newParticle.t = (float)rand()/(float)RAND_MAX;
	
	ModelerUserInterface* ui = ModelerUserInterface::getInstance();
	newParticle.createTime = ui->getTime();

	getParticleSystem()->particles.push_back(newParticle);

}