#ifndef MODEL_H
#define MODEL_H

#include <cmath>
#include "vec.h"
#include "properties.h"
#include "vault.h"
#include "particleSystem.h"
#include "mat.h"

/** Forward-declare ICamera because it's used in Model yet is a Model. */
class ICamera;

/**
 * Forward-declare ParticleSystem because it's used in Model yet it's a
 * Model.
 */
class ParticleSystem;

/**
 * You will extend this class to create your model.
 */
class Model {
protected:
	/**
	 * A GroupProperty containing your model's name and a list of its
	 * properties.  Modeler will obtain this object using the getProperties()
	 * method, so it can link controls to your properties.
	 */
	GroupProperty properties;

public:
	Model(const char* name);

	/**
	 * This method returns the model's list of properties.
	 */
	GroupProperty* getProperties();

	/**
	 * Override this method to draw your model to the current OpenGL context.
	 */
	virtual void draw();

	/**
	 * (Optional) Override this method to update your animation counters.
	 */
	virtual void tick();

	/**
	 * Override this method to load your textures, shaders, and anything else
	 * that requires an OpenGL context.
	 */
	virtual void load();

	/**
	 * Override this method to return the camera you want to look through.
	 */
	virtual ICamera* getCamera();

	/**
	 * Override this method to return the particle system you want to control.
	 */
	virtual ParticleSystem* getParticleSystem();
};

/**
 * This class is abstract.  Don't mess with it.
 */
class Light : public Model {
protected:
	GLuint lightNumber;

public:
	RGBProperty diffuse, specular, ambient;
	BooleanProperty isOn;
	RangeProperty lightX, lightY, lightZ;

	Light(const char* name, GLuint lightNumber,
				float x = 0, float y = 0, float z = 0,
				float dr = 1, float dg = 1, float db = 1,
				float sr = 1, float sg = 1, float sb = 1,
				float ar = .1, float ag = .1, float ab = .1);

	void draw(bool directional);
};

/**
 * A point light.
 */
class PointLight : public Light {
public:
	RangeProperty attenA, attenB, attenC;

	PointLight(const char* name, GLuint lightNumber,
				float x = 0, float y = 0, float z = 0,
				float dr = 1, float dg = 1, float db = 1,
				float sr = 1, float sg = 1, float sb = 1,
				float ar = .1, float ag = .1, float ab = .1,
				float attA = .5, float attB = .7, float attC = 0);

	void draw();
};

/**
 * A directional light.
 */
class DirectionalLight : public Light {
public:
	DirectionalLight(const char* name, GLuint lightNumber,
				float x = 0, float y = 0, float z = 0,
				float dr = 1, float dg = 1, float db = 1,
				float sr = 1, float sg = 1, float sb = 1);

	void draw();
};

class CuriosityRover : public Model {
protected:
	RangeProperty size;

	RangeProperty roverX;
	RangeProperty roverY;
	RangeProperty roverZ;

	RangeProperty portFrontRodRotation;
	RangeProperty portCenterRodRotation;
	RangeProperty portBackFrontRodRotation;
	RangeProperty portBackBackRodRotation;

	RangeProperty starboardFrontRodRotation;
	RangeProperty starboardCenterRodRotation;
	RangeProperty starboardBackFrontRodRotation;
	RangeProperty starboardBackBackRodRotation;

	RangeProperty portFrontWheelSwivel;
	RangeProperty portCenterWheelSwivel;
	RangeProperty portBackWheelSwivel;
	RangeProperty starboardFrontWheelSwivel;
	RangeProperty starboardCenterWheelSwivel;
	RangeProperty starboardBackWheelSwivel;

	RangeProperty portFrontWheelSpin;
	RangeProperty portCenterWheelSpin;
	RangeProperty portBackWheelSpin;
	RangeProperty starboardFrontWheelSpin;
	RangeProperty starboardCenterWheelSpin;
	RangeProperty starboardBackWheelSpin;

	RangeProperty globalWheelSpin;

	RangeProperty cameraMastSwivel;
	RangeProperty cameraTilt;
	RangeProperty cameraBlink;

	RangeProperty upperArmSwivel;
	RangeProperty upperArmTilt;

	RangeProperty lowerArmTilt;

	RangeProperty handTilt;

	RangeProperty firingLaser;
	RangeProperty dusting;

	ParticleSystem ps;

public:
    CuriosityRover(const char*, GLdouble, GLdouble, GLdouble, GLdouble);
    void draw();
	void load();
	//void tick();
	void spawnParticles(Mat4f, int);
	void addParticleStartingAt(Vec4f, int, Vec3f);
	ParticleSystem* getParticleSystem() { return &ps; }
private:
	GLdouble x, y, z;
	const char* name;

	Texture2D chassisTex;
	Texture2D bogieTex;
	Texture2D wheelTex;
	Texture2D RTGTex;
	Texture2D cameraMastTex;
	Texture2D cameraTex0;
	Texture2D cameraTex1;
	Texture2D cameraTex2;
	Texture2D cameraTex3;
	Texture2D cameraTex4;
	Texture2D cameraTex5;
	Texture2D cameraTex6;
	Texture2D cameraTex7;
	Texture2D cameraTex8;
	Texture2D armTex;
	Texture2D handTex;

	void drawChassis();
    void drawRockerBogies(Mat4f);
	void drawWheel(Mat4f);
    void drawCameraMast();
	void drawRTG();
    void drawArm(Mat4f);

};

#endif // MODEL_H