#pragma warning(disable : 4786)

#include "particleSystem.h"
#include "modelerui.h"

#include <cstdio>
#include <cstdlib>
#include <cassert>
#include <cmath>
#include <time.h>
//#include <limits>

using namespace std;

static float prevT;

/***************
 * Constructors
 ***************/

ParticleSystem::ParticleSystem() : dustParticleTexture("dustParticleTex.png"), laserParticleTexture("laserParticleTex.png"), restitution("Restitution", 0.0f, 2.0f, 1.0f, 0.1f), friction("Friction", 0.0f, 50.0f, 20.0f, 0.1f)
{

	// Leave these here; the UI needs them to work correctly.
	dirty = false;
	simulate = false;

	running = false;

}


/*************
 * Destructor
 *************/

ParticleSystem::~ParticleSystem() 
{
	// TODO
}


/******************
 * Simulation fxns
 ******************/

/** Start the simulation */
void ParticleSystem::startSimulation(float t)
{
	running = true;

	dustParticleTexture.load();
	laserParticleTexture.load();

	// These values are used by the UI ...
	// negative bake_end_time indicates that simulation
	// is still progressing, and allows the
	// indicator window above the time slider
	// to correctly show the "baked" region
	// in grey.
	bake_end_time = -1;
	simulate = true;
	dirty = true;

}

/** Stop the simulation */
void ParticleSystem::stopSimulation(float t)
{
	running = false;
	particles.clear();

	// These values are used by the UI
	simulate = false;
	dirty = true;

}

/** Reset the simulation */
void ParticleSystem::resetSimulation(float t)
{
	particles.clear();
	running = false;

	// These values are used by the UI
	simulate = false;
	dirty = true;

}

/** Compute forces and update particles **/
void ParticleSystem::computeForcesAndUpdateParticles(float t)
{
	float groundY = -2.f;

	for (int i = 0; i < particles.size(); i++)
	{
		float deltaT;

		// How long has it been since your last update?
		if (particles[i].lastUpdateTime < 0)
		{
			// First update; do nothing (but set the particle
			// so that it'll catch up next frame)
			deltaT = 0;
			particles[i].lastUpdateTime = particles[i].createTime;
			return;
		}
		else
		{
			deltaT = t - particles[i].lastUpdateTime;
			particles[i].lastUpdateTime = t;
		}

		// If we're dealing with dust, apply forces
		if (particles[i].type == 0)
		{
			// Apply gravitational force to velocity
			Vec3f marsGravityForce(0.f, -0.8f, 0.f);
			
			// Apply pressure drag to particles; note that here, dragCoeff
			// represents embodies the actual drag coefficient, as well
			// as atmospheric density and the exposed object area
			// Note that pressure drag differs from viscous drag and
			// usually has a greater effect on objects in the air...
			float dragCoeff = 0.005f;
			Vec3f dragForce(dragCoeff * particles[i].velocity[0] * particles[i].velocity[0], dragCoeff * particles[i].velocity[1] * particles[i].velocity[1], dragCoeff * particles[i].velocity[2] * particles[i].velocity[2]);
			dragForce = -dragForce; // Acts opposite to the velocity direction

			Vec3f Fnet;

			// Don't apply gravity and drag at a very low height; this
			// will help particles to not fall through the floor
			if (particles[i].position[1] < groundY + 0.01f)
			{
				Fnet = Vec3f(0.f, 0.f, 0.f);
			}
			else
			{
				Fnet = marsGravityForce + dragForce;
			}
	

			Vec3f acc = Fnet / particles[i].mass;
			particles[i].velocity[0] += acc[0] * deltaT;
			particles[i].velocity[1] += acc[1] * deltaT;
			particles[i].velocity[2] += acc[2] * deltaT;

			// Finally, apply friction to particles that are touching the ground (and which are moving)
			if ((particles[i].position[1] < groundY + 0.1f) && (particles[i].velocity.length() > 0.01f))
			{
				float muK = friction.getValue(); // kinetic friction coefficient (exaggerated to get the right results)
				float frictionForceMag = muK * particles[i].mass * 0.8f;
				Vec3f frictionForceDir = -particles[i].velocity;
				frictionForceDir.normalize();
				Vec3f frictionForce = frictionForceMag * frictionForceDir;

				// Friction will not reverse the direction of an object;
				// it will only slow it down
				if (particles[i].velocity[0] + (frictionForce[0] / particles[i].mass) * deltaT > 0)
				{
					particles[i].velocity[0] += (frictionForce[0] / particles[i].mass) * deltaT;
				}
				if (particles[i].velocity[2] + (frictionForce[2] / particles[i].mass) * deltaT > 0)
				{
					particles[i].velocity[2] += (frictionForce[2] / particles[i].mass) * deltaT;
				}
			}

			// If the particle's moving extremely slowly, just stop it
			if (particles[i].velocity.length() < 0.01f)
			{
				particles[i].velocity[0] = 0.f;
				particles[i].velocity[1] = 0.f;
				particles[i].velocity[2] = 0.f;
			}

		}

		// Apply velocity to positions
		particles[i].position[0] += particles[i].velocity[0] * deltaT;
		particles[i].position[1] += particles[i].velocity[1] * deltaT;
		particles[i].position[2] += particles[i].velocity[2] * deltaT;

		// If we've gone past the ground (and we're going down), rebound!
		float epsilon = 0.1;
		if ((particles[i].position[1] < groundY + epsilon)&&(particles[i].velocity[1] < 0.f))
		{
			particles[i].velocity[1] = -particles[i].velocity[1] * restitution.getValue();
		}
	}




	// Debugging info
	/*if( t - prevT > .08 )
		printf("(!!) Dropped Frame %lf (!!)\n", t-prevT);*/
	prevT = t;
}

// Cleans up the oldest particles
void ParticleSystem::cleanup()
{
	while(particles.size() > 200)
	{
		std::vector<Particle>::iterator it = particles.begin();
		particles.erase(it);
	}

}

/** Render particles */
void ParticleSystem::drawParticles(float t)
{
	for (int i = 0; i < particles.size(); i++)
	{
		float centerX = particles[i].position[0];
		float centerY = particles[i].position[1];
		float centerZ = particles[i].position[2];
		

		if (particles[i].type == 0) // dust
		{
			dustParticleTexture.use();

			// Draw a little particle at the position
			glTranslatef(centerX, centerY, centerZ);
			drawSphere(0.05f);

			glTranslatef(-centerX, -centerY, -centerZ);
		}
		else // laser
		{
			laserParticleTexture.use();

			glTranslatef(centerX, centerY, centerZ);
			// Scale in the direction of the velocity
			Vec3f scaling = particles[i].velocity;
			scaling.normalize();
			scaling *= 7.f;

			glScalef(abs(scaling[0]) + 1.f, abs(scaling[1]) + 1.f, abs(scaling[2]) + 1.f);
		
			drawSphere(0.075f);
		
			// Scale and translate back
			glScalef(1.f / (abs(scaling[0]) + 1.f), 1.f / (abs(scaling[1]) + 1.f), 1.f / (abs(scaling[2]) + 1.f));
			glTranslatef(-centerX, -centerY, -centerZ);
		}

	}
}




/** Adds the current configuration of particles to
  * your data structure for storing baked particles **/
void ParticleSystem::bakeParticles(float t) 
{
	// TODO (baking is extra credit)
}

/** Clears out your data structure of baked particles */
void ParticleSystem::clearBaked()
{
	// TODO (baking is extra credit)
}




